////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Elektrobit Automotive GmbH
// Alle Rechte vorbehalten. All Rights Reserved.
//
// Information contained herein is subject to change without notice.
// Elektrobit retains ownership and all other rights in the software and each
// component thereof.
// Any reproduction of the software or components thereof without the prior
// written permission of Elektrobit is prohibited.
////////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <gtf/datapool/Value.h>
#include <gtf/datapool/ListValue.h>
#include <gtf/datapool/ScalarValue.h>
#include <gtf/eventsystem/ScalarValue.h>
#include <gtf/resources/ImageResource.h>

#include "HmiAppInteractionDemo.h"
#include "MandelbrotImage.h"
#include "local_datapool.h"
#include "global_events.h"
#include <gtf/configuration/Settings.h>
#include <gtf/configuration/ScalarValue.h>

applicationInteractionDemo::ApplicationInteractionDemo::ApplicationInteractionDemo(gtf::dependencyresolver::DependencyContainerHandle& container_, gtf::osal::MainTaskDispatcherHandle& taskDispatcher_, gtf::datapool::DatapoolHandle & datapool_, gtf::eventsystem::global::EventQueueHandle& eventQueue_)
:container(container_)
, taskDispatcher(taskDispatcher_)
, datapool(datapool_)
, eventQueue(eventQueue_)
, loopCounter(0)

{
    // setup the datapool and event processing task
    processTask.setExecuteCallback(gtf::utils::Delegate<void()>::create<ApplicationInteractionDemo, &ApplicationInteractionDemo::processItems>(this));

    // setup the loop processing task
    processLoopTask.setExecuteCallback(gtf::utils::Delegate<void()>::create<ApplicationInteractionDemo, &ApplicationInteractionDemo::processLoop>(this));
}

void applicationInteractionDemo::ApplicationInteractionDemo::loadMandelBrot()
{
    if (!resourcer.valid())
    {
        return;
    }

    if (!resourcer->isInitialized())
    {
        return;
    }

    if (memImageType.isValid()
        && mandelbrotImageType.isValid())
    {
        return;
    }
    // setup the resourcer to support our image format
    // resolve the resource data type of GtfMemImage
    memImageType = resourcer->resolveResourceFormat(GTF_RESOURCE_FORMAT_GTF_MEM_IMAGE);
    if (memImageType.isValid())
    {
        imageConverter.setDestinationFormat(memImageType);

        // add our own new resource data type, used for ManelbrotImage objects
        resourcer->addResourceFormat(gtf::type::TypeId::eTypeId_image, MANDELBROT_IMAGE_FORMAT);
        mandelbrotImageType = resourcer->resolveResourceFormat(MANDELBROT_IMAGE_FORMAT);

        if (mandelbrotImageType.isValid())
        {
            // set a converter from m_mandelbrotImageType to m_gtfMemImageType
            resourcer->setLoader(&imageConverter,
                                 mandelbrotImageType,
                                 memImageType);
        }
    }
}

void applicationInteractionDemo::ApplicationInteractionDemo::run()
{
    // retrieve a handle to a Resourcer
    resourcer = container->get<gtf::resourcer::Resourcer>();

    if (!resourcer)
    {
        return;
    }

    //build an imageFilePath to an external image
    imagePath = getImagePath();

    initializeDatapool();

    // subscribe to the event, in order to recieve a notification in the handleEvents function
    eventQueue->subscribe(gtf::eventsystem::SubscriptionType::Event
        , GLOBAL_EVENT_GRP_ID_evInterplayOut
        , GLOBAL_EVENT_MSG_ID_evInterplayOut
        , this
        , gtf::utils::Delegate<void(gtf::eventsystem::global::EventHandle)>::create<ApplicationInteractionDemo, &ApplicationInteractionDemo::handleEvents>(this));

    // start the loop task
    taskDispatcher->performTaskDelayed(processLoopTask, 500);
}

void applicationInteractionDemo::ApplicationInteractionDemo::stop()
{
    //unsubscrie for event
    eventQueue->unsubscribe(gtf::eventsystem::SubscriptionType::All
        , GLOBAL_EVENT_GRP_ID_evInterplayOut
        , GLOBAL_EVENT_MSG_ID_evInterplayOut
        , this);

    //cancel ongoing tasks
    if (taskDispatcher)
    {
        taskDispatcher->cancelTask(processTask);
        taskDispatcher->cancelTask(processLoopTask);
        taskDispatcher->stop();
        taskDispatcher.reset();
    }

    //reset handles
    datapool.reset();
    resourcer.reset();
}


void applicationInteractionDemo::ApplicationInteractionDemo::handleEvents(gtf::eventsystem::global::EventHandle event)
{
    if (!event || event->getParameterCount() == 0)
    {
        return;
    }

    // declare a handle to the event's parameter in order to get it's value
    gtf::eventsystem::IntegerValueHandle myParameterHandle = event->getParameter<gtf::eventsystem::IntegerValue>(0);

    if (myParameterHandle.valid())
    {
        int32_t myParameter = myParameterHandle->get();

        // create the answer event to be sent to the model
        gtf::eventsystem::global::EventHandle responseEvent = eventQueue->createEvent(GLOBAL_EVENT_GRP_ID_evInterplayIn, GLOBAL_EVENT_MSG_ID_evInterplayIn);

        if (responseEvent.valid())
        {
            // append the read parameter
            responseEvent->addParameter<gtf::eventsystem::IntegerValue>()->set(myParameter);

            // send the answer event to the model
            eventQueue->sendEvent(responseEvent);
        }
    }
}

void applicationInteractionDemo::ApplicationInteractionDemo::initializeDatapool()
{
    //get the path to the datapool description file, in order to load the datapool items
    gtf::configuration::PathValueHandle descriptionFile = getDatapoolDescription();
    if (!descriptionFile.valid())
    {
        return;
    }

    datapool->getLoader().load(descriptionFile->get());
    datapool->setInvoker(gtf::utils::Delegate<void()>::create<ApplicationInteractionDemo, &ApplicationInteractionDemo::invoker>(this));
    datapool->update();
    // read the initial values of all items in order to receive further notifications
    readAllDatapoolItems();
}

gtf::configuration::PathValueHandle applicationInteractionDemo::ApplicationInteractionDemo::getDatapoolDescription()
{
    // retrieve settings handle to provide access to the configuration items
    gtf::configuration::SettingsHandle settings = container->get<gtf::configuration::Settings>();
    if (!settings.valid())
    {
        return gtf::configuration::PathValueHandle();
    }

    // get the path to the model
    gtf::configuration::StringScalarHandle basePath = settings->getItem<gtf::configuration::StringScalar>("gtf.model.default.path");
    if (!basePath.valid())
    {
        return gtf::configuration::PathValueHandle();
    }

    // retrieve the configuration value of the "datapoolFile" item in the model.json
    gtf::configuration::PathValueHandle filePath = settings->getItem<gtf::configuration::PathValue>("datapoolFile");
    if (!filePath.valid())
    {
        return gtf::configuration::PathValueHandle();
    }

    // set the complete path to the datapool description file
    filePath->setBasePath(basePath->get());
    return filePath;
}

void applicationInteractionDemo::ApplicationInteractionDemo::readAllDatapoolItems()
{
    gtf::container::Array<uint32_t> items;

    bool canRetreiveItems = datapool->getAllItems(items);
    if (!canRetreiveItems)
    {
        return;
    }

    for (gtf::container::Array<uint32_t>::Iterator it = items.begin(), e = items.end(); it != e; ++it)
    {
        gtf::datapool::ValueHandle value = datapool->getItemByIndex<gtf::datapool::Value>(*it);

        // avoid reading items without a valid value, e.g. scripts
        if (!value)
        {
            continue;
        }

        value->read();
    }
}

void applicationInteractionDemo::ApplicationInteractionDemo::invoker()
{
    //start the datapool and event processing task
    taskDispatcher->performTask(processTask);
}

void applicationInteractionDemo::ApplicationInteractionDemo::processItems()
{
    gtf::container::Array<uint32_t> notifications;

    //update the datapool items
    datapool->update();
    bool canRetreiveNotifications = datapool->getNotifications(notifications);

    if (!canRetreiveNotifications)
    {
        return;
    }

    for (gtf::container::Array<uint32_t>::Iterator it = notifications.begin(), e = notifications.end(); it != e; ++it)
    {
        switch (*it)
        {
        case LOCAL_DP_ID_dpInterplayOut:
        {
            // read the value from DP_ID_dpInterplayOut
            int32_t itemValue = 0;
            gtf::datapool::IntegerValueHandle value = datapool->getItemByIndex<gtf::datapool::IntegerValue>(*it);
            if (value.valid())
            {
                value->get(itemValue);
            }

            // write the value to DP_ID_dpInterplayIn
            gtf::datapool::IntegerValueHandle responseValue = datapool->getItemByIndex<gtf::datapool::IntegerValue>(LOCAL_DP_ID_dpInterplayIn);
            if (responseValue.valid())
            {
                responseValue->set(itemValue);
            }
            break;
        }

        // update the windowed list when the datapool items bellow change
        case LOCAL_DP_ID_dpWindowPosOut:
        case LOCAL_DP_ID_dpWindowSizeOut:
            updateWindowList();
            break;
        default:
        {
            break;
        }
        }
    }
    // commit all changed properties
    datapool->commit();

    //release all event from the queue
    eventQueue->dispatch();
}

void applicationInteractionDemo::ApplicationInteractionDemo::updateWindowList()
{
    uint32_t listLength = 999;
    int32_t windowPosition_value = 0;
    char tmpBufferUTF8[50];

    // read the window position
    gtf::datapool::IntegerValueHandle windowPosition = datapool->getItemByIndex<gtf::datapool::IntegerValue>(LOCAL_DP_ID_dpWindowPosOut);
    if (windowPosition.valid())
    {
        windowPosition->get(windowPosition_value);
    }

    // read the window size
    gtf::datapool::IntegerValueHandle windowSize = datapool->getItemByIndex<gtf::datapool::IntegerValue>(LOCAL_DP_ID_dpWindowSizeOut);
    if (!windowSize.valid())
    {
        return;
    }

    // set a handle to the List
    gtf::datapool::StringListHandle windowList = datapool->getItemByIndex<gtf::datapool::StringList>(LOCAL_DP_ID_dpWindowListIn);
    if (!windowList.valid())
    {
        return;
    }

    // set the type of the list to Windowed
    windowList->reset(gtf::datapool::List::Mode::Windowed);

    // resize the list
    windowList->resize(listLength);

    //create a window
    gtf::datapool::WindowHandle windowHandle = windowList->createWindow();
    if (!windowHandle.valid())
    {
        return;
    }

    // set the corresponding position and size of the window
    windowHandle->set(windowPosition_value, windowSize->get());

    uint32_t tmpLength = MIN(listLength, static_cast<uint32_t>(windowPosition_value + windowSize->get()));

    // update the window content
    for (uint32_t index = windowPosition_value; index < tmpLength; index++)
    {
        switch (index)
        {
        case 0:
        {
            windowList->setElement(index, GTF_VERSION_STRING);
            break;
        }
        case 1:
        {
            windowList->setElement(index, GTF_VERSION_COMPANY);
            break;
        }

        case 2:
        {
            windowList->setElement(index, GTF_PRODUCT_NAME_STRING);
            break;
        }
        default:
        {
            sprintf(tmpBufferUTF8, "line %d", index);
            windowList->setElement(index, tmpBufferUTF8);
            break;
        }
        }
    }
}

void applicationInteractionDemo::ApplicationInteractionDemo::changeBooleanValues()
{
    bool value = ((loopCounter % 2) != 0);

    gtf::datapool::BoolValueHandle booleanValue = datapool->getItemByIndex<gtf::datapool::BoolValue>(LOCAL_DP_ID_dpUpdatesBoolIn);
    if (booleanValue.valid())
    {
        booleanValue->set(value);
    }
}

void applicationInteractionDemo::ApplicationInteractionDemo::changeIntValues()
{
    int32_t value = static_cast<int32_t>(loopCounter);

    gtf::datapool::IntegerValueHandle integerValue = datapool->getItemByIndex<gtf::datapool::IntegerValue>(LOCAL_DP_ID_dpUpdatesInt32In);
    if (integerValue)
    {
        integerValue->set(value);
    }
}

void applicationInteractionDemo::ApplicationInteractionDemo::changeStringValues()
{
    char tmpBufferUTF8[50];
    sprintf(tmpBufferUTF8, "0x%X", loopCounter);

    gtf::datapool::StringValueHandle stringValue = datapool->getItemByIndex<gtf::datapool::StringValue>(LOCAL_DP_ID_dpUpdatesStringIn);
    if (stringValue)
    {
        stringValue->set(tmpBufferUTF8);
    }
}

void applicationInteractionDemo::ApplicationInteractionDemo::loadExternalImage()
{
    if (!imagePath.valid())
    {
        return;
    }
    // create a new image handle
    gtf::resources::ImageResourceHandle image(resourcer->createResource(gtf::type::TypeId::eTypeId_image, gtf::resourcer::Resourcer::TiedToResource, resourcer->resolveResourceFormat(GTF_RESOURCE_FORMAT_GTF_IMAGE_RESOURCE)), &*resourcer);

    gtf::resourcer::ResourceLoader::resourcerError error;

    // set pointer to a ImageResource object attached to the resource
    gtf::smartptr::RefCountedPtr<gtf::resources::ImageResource> imageResource = image.getImageResource(error);

    if (imageResource)
    {
        //set the path of the resource
        imageResource->setPath(imagePath->get());
    }

    // write the image handle into the datapool
    gtf::datapool::ResourceValueHandle imageHandle = datapool->getItemByIndex<gtf::datapool::ResourceValue>(LOCAL_DP_ID_dpUpdatesExternalImageIn);

    if (imageHandle)
    {
        imageHandle->set(image);
    }
}

void applicationInteractionDemo::ApplicationInteractionDemo::loadDynamicImage()
{
    if (mandelbrotImageType.isValid())
    {
        gtf::resources::ImageResourceHandle image(resourcer->createResource(gtf::type::TypeId::eTypeId_image),
                                                  &*resourcer);

        image.attachData(imageCreator.createMandelbrotImage(mandelbrotImageType, loopCounter));

        // create a ResourceValueHandle for loading the image into the datapool
        gtf::datapool::ResourceValueHandle imageHandle = datapool->getItemByIndex<gtf::datapool::ResourceValue>(LOCAL_DP_ID_dpUpdatesImageIn);

        if (imageHandle)
        {
            imageHandle->set(image);
        }
    }
}

void applicationInteractionDemo::ApplicationInteractionDemo::processLoop()
{
    loadMandelBrot();

    loopCounter++;

    changeBooleanValues();
    changeIntValues();
    changeStringValues();

    loadExternalImage();

    loadDynamicImage();

    datapool->commit();
    taskDispatcher->performTaskDelayed(processLoopTask, 500);
}

gtf::configuration::PathValueHandle applicationInteractionDemo::ApplicationInteractionDemo::getImagePath()
{
    // load the an external image extern.png
    gtf::configuration::SettingsHandle settings = container->get<gtf::configuration::Settings>();
    if (!settings.valid())
    {
        return gtf::configuration::PathValueHandle();
    }

    gtf::configuration::StringScalarHandle basePath = settings->getItem<gtf::configuration::StringScalar>("gtf.model.default.path");
    if (!basePath.valid())
    {
        return gtf::configuration::PathValueHandle();
    }
    // retrieve the configuration value of the "ImageFile" item in the model.json
    gtf::configuration::PathValueHandle filePath = settings->getItem<gtf::configuration::PathValue>("ImageFile");
    if (!filePath.valid())
    {
        return gtf::configuration::PathValueHandle();
    }

    //set the complete path to the image
    filePath->setBasePath(basePath->get());
    return filePath;
}
