////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Elektrobit Automotive GmbH
// Alle Rechte vorbehalten. All Rights Reserved.
//
// Information contained herein is subject to change without notice.
// Elektrobit retains ownership and all other rights in the software and each
// component thereof.
// Any reproduction of the software or components thereof without the prior
// written permission of Elektrobit is prohibited.
////////////////////////////////////////////////////////////////////////////////

#include "Mandelbrot2MemImageConverter.h"
#include "MandelbrotImage.h"
#include <cmath>
#include <gtf/resources/ImageResource.h>

Mandelbrot2MemImageConverter::Mandelbrot2MemImageConverter()
{
    for(uint32_t i = 0; i != RainbowGradientSize ; ++i)
    {
        uint32_t red   = static_cast<uint8_t>(sin(0.03f *float(i) + 0.0f)*127.0f + 128.0f);
        uint32_t blue  = static_cast<uint8_t>(sin(0.03f *float(i) + 2.0f)*127.0f + 128.0f);
        uint32_t green = static_cast<uint8_t>(sin(0.03f *float(i) + 4.0f)*127.0f + 128.0f);
        rainbow[i] = green + (blue << 8) + (red<<16);
    }
}

void Mandelbrot2MemImageConverter::setDestinationFormat(gtf::resourcer::ResourceFormat format)
{
    m_destinationFormat = format;
}

gtf::smartptr::RefCountedPtr<gtf::resourcer::ResourceData>
Mandelbrot2MemImageConverter::load(gtf::smartptr::RefCountedPtr<gtf::resourcer::ResourceData> const& baseData, resourcerError &errorOut)
{
    gtf::smartptr::RefCountedPtr<MandelbrotImage> mandelbrotImage = baseData.downcast<MandelbrotImage>();
    if(!mandelbrotImage)
    {
        errorOut = gtf::resourcer::ResourceLoader::eResourcerCriticalError;
        return NULL;
    }

    gtf::resources::MemImage *returnImage = new gtf::resources::MemImage(m_destinationFormat, this);
    returnImage->m_colorSpace = gtf::resources::MemImage::eColorSpace_rgb;
    returnImage->m_channelDepth = gtf::resources::MemImage::eImageLoader_RGB_to_BGR888;
    returnImage->imageData = new HeapImageData(mandelbrotImage->getWidth() * mandelbrotImage->getHeight() * 3);
    returnImage->m_width = mandelbrotImage->getWidth();
    returnImage->m_height = mandelbrotImage->getHeight();
    returnImage->m_stride = mandelbrotImage->getWidth()*3;

    for(uint32_t i = 0; i != mandelbrotImage->getHeight(); ++i)
    {
        MandelbrotImage::value_type *srcRow = (*mandelbrotImage)[i];
        uint8_t *destRow = returnImage->row_ptr(i);
        for(uint32_t j = 0; j != mandelbrotImage->getWidth(); ++j)
        {
            if(srcRow[j]> 0.0)
            {
                uint32_t index1 = static_cast<uint32_t>(srcRow[j]*50)%RainbowGradientSize;
                uint32_t col_1 = rainbow[index1];
                destRow[j*3 + 0] = static_cast<uint8_t>( col_1 & 0xFF);
                destRow[j*3 + 1] = static_cast<uint8_t>((col_1 & 0xFF00 ) >> 8);
                destRow[j*3 + 2] = static_cast<uint8_t>((col_1 & 0xFF0000 ) >> 16);
            }
            else
            {
                destRow[j*3 + 0] = 0;
                destRow[j*3 + 1] = 0;
                destRow[j*3 + 2] = 0;
            }
        }
    }

    errorOut = gtf::resourcer::ResourceLoader::eResourcerSuccess;
    return returnImage;
}

void Mandelbrot2MemImageConverter::unload(gtf::resourcer::ResourceData const* destination)
{
    if(destination == NULL)
        return;

    // In a setup with an OpenGL|ES this method is called right after loading the image to
    // texture memory, since the GtfMemImage is only an intermediate conversion result.

    // In a setup with a software renderer the GtfMemImage will be unloaded when the
    // resource handle used to store the image is no longer referenced.
    gtf::resources::MemImage const *memImage = static_cast<gtf::resources::MemImage const*>(destination);
    delete memImage;
}
