////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Elektrobit Automotive GmbH
// Alle Rechte vorbehalten. All Rights Reserved.
//
// Information contained herein is subject to change without notice.
// Elektrobit retains ownership and all other rights in the software and each
// component thereof.
// Any reproduction of the software or components thereof without the prior
// written permission of Elektrobit is prohibited.
////////////////////////////////////////////////////////////////////////////////
#include "DatapoolApp.h"

#include <gtf/configuration/Settings.h>
#include <gtf/configuration/ScalarValue.h>
#include <gtf/configuration/PathValue.h>
#include <gtf/datapool/ScalarValue.h>

#include "local_datapool.h"

datapoolplugin::DatapoolApplication::DatapoolApplication(gtf::dependencyresolver::DependencyContainerHandle& container_, gtf::osal::MainTaskDispatcherHandle taskDispatcher_, gtf::datapool::DatapoolHandle datapool_)
: container(container_)
, datapool(datapool_)
, taskDispatcher(taskDispatcher_)
{
    // setup the datapool processing task
    processDatapoolTask.setExecuteCallback(gtf::utils::Delegate<void()>::create<DatapoolApplication, &DatapoolApplication::processDatapool>(this));
}

void datapoolplugin::DatapoolApplication::initializeDatapool()
{
    //get the path to the datapool description file, in order to load the datapool items
    gtf::configuration::PathValueHandle descriptionFile = getDatapoolDescription();
    if (!descriptionFile.valid())
    {
        return;
    }

    datapool->getLoader().load(descriptionFile->get());
    datapool->setInvoker(gtf::utils::Delegate<void()>::create<DatapoolApplication, &DatapoolApplication::invoker>(this));
    datapool->update();

    // read the initial values of all items in order to receive further notifications
    readAllDatapoolItems();
}

gtf::configuration::PathValueHandle datapoolplugin::DatapoolApplication::getDatapoolDescription()
{
    // retrieve settings handle to provide access to the configuration items
    gtf::configuration::SettingsHandle settings = container->get<gtf::configuration::Settings>();
    if (!settings.valid())
    {
        return gtf::configuration::PathValueHandle();
    }

    // get the path to the model
    gtf::configuration::StringScalarHandle basePath = settings->getItem<gtf::configuration::StringScalar>("gtf.model.default.path");
    if (!basePath.valid())
    {
        return gtf::configuration::PathValueHandle();
    }

    // retrieve the configuration value of the "datapoolFile" item in the model.json
    gtf::configuration::PathValueHandle filePath = settings->getItem<gtf::configuration::PathValue>("datapoolFile");
    if (!filePath.valid())
    {
        return gtf::configuration::PathValueHandle();
    }

    // set the complete path to the datapool description file
    filePath->setBasePath(basePath->get());
    return filePath;
}

void datapoolplugin::DatapoolApplication::readAllDatapoolItems()
{
    gtf::container::Array<uint32_t> items;

    bool canRetreiveItems = datapool->getAllItems(items);
    if (!canRetreiveItems)
    {
        return;
    }

    for (gtf::container::Array<uint32_t>::Iterator it = items.begin(), e = items.end(); it != e; ++it)
    {
        gtf::datapool::ValueHandle value = datapool->getItemByIndex<gtf::datapool::Value>(*it);

        // avoid reading items without a valid value, e.g. scripts
        if (!value)
        {
            continue;
        }

        value->read();
    }
}

void datapoolplugin::DatapoolApplication::run()
{
    initializeDatapool();
}

void datapoolplugin::DatapoolApplication::stop()
{
    // reset datapool handle
    datapool.reset();
}

void datapoolplugin::DatapoolApplication::invoker()
{
    //start the datapool processing task
    taskDispatcher->performTask(processDatapoolTask);
}

void datapoolplugin::DatapoolApplication::processListElements(gtf::datapool::IntegerListHandle list_value)
{
    //get response list handle
    gtf::datapool::IntegerListHandle response_list = datapool->getItemByIndex<gtf::datapool::IntegerList>(LOCAL_DP_ID_myReceivedList);

    if (!response_list.valid())
    {
        return;
    }

    response_list->resize(0);

    uint32_t size = list_value->size();

    for (uint32_t i = 0; i < size; i++)
    {
        if (list_value->getElement(i) == 0)
        {
            break;
        }
        int32_t itemValue = list_value->getElement(i);
        response_list->append(2 * itemValue);
    }
}

void datapoolplugin::DatapoolApplication::processDatapool()
{
    gtf::container::Array<uint32_t> notifications;

    //update the datapool items
    datapool->update();
    bool canRetreiveNotifications = datapool->getNotifications(notifications);

    if (!canRetreiveNotifications)
    {
        return;
    }

    for (gtf::container::Array<uint32_t>::Iterator it = notifications.begin(), e = notifications.end(); it != e; ++it)
    {
        switch (*it)
        {
            case LOCAL_DP_ID_mySentValue:
            {
                // read the value from DP_ID_mySentValue
                gtf::datapool::IntegerValueHandle valueHandle = datapool->getItemByIndex<gtf::datapool::IntegerValue>(*it);
                int32_t itemValue = 0;

                if (valueHandle.valid())
                {
                    valueHandle->get(itemValue);
                }
                // write the value to DP_ID_myReceivedValue
                gtf::datapool::IntegerValueHandle responseValueHandle = datapool->getItemByIndex<gtf::datapool::IntegerValue>(LOCAL_DP_ID_myReceivedValue);

                if (responseValueHandle.valid())
                {
                    responseValueHandle->set(itemValue);
                }
                break;
            }
            case LOCAL_DP_ID_MySentList:
            {
                // read the value from list LOCAL_DP_ID_MySentList
                gtf::datapool::IntegerListHandle list_valueHandle = datapool->getItemByIndex<gtf::datapool::IntegerList>(*it);

                if (list_valueHandle.valid())
                {
                    //fill list elements
                    processListElements(list_valueHandle);
                }
                break;
            }
      default:
      {
        break;
      }
        }
    }
    datapool->commit();
}

