////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Elektrobit Automotive GmbH
// Alle Rechte vorbehalten. All Rights Reserved.
//
// Information contained herein is subject to change without notice.
// Elektrobit retains ownership and all other rights in the software and each
// component thereof.
// Any reproduction of the software or components thereof without the prior
// written permission of Elektrobit is prohibited.
////////////////////////////////////////////////////////////////////////////////
#include "WindowListApp.h"
#include <gtf/configuration/ScalarValue.h>
#include <gtf/configuration/Settings.h>
#include "local_datapool.h"
#include <gtf/datapool/ScalarValue.h>
#include "stdio.h"
#include <gtf/configuration/PathValue.h>

using namespace gtf;
using namespace datapool;

windowlistplugin::WindowListApplication::WindowListApplication(dependencyresolver::DependencyContainerHandle& container_, osal::MainTaskDispatcherHandle & taskDispatcher_, DatapoolHandle & datapool_)
: container(container_)
, datapool(datapool_)
, taskDispatcher(taskDispatcher_)
{
    // setup the datapool processing task
    processDatapoolTask.setExecuteCallback(utils::Delegate<void()>::create<WindowListApplication, &WindowListApplication::processDatapoolItems>(this));
}

void windowlistplugin::WindowListApplication::run()
{
    initializeDatapool();
}

void windowlistplugin::WindowListApplication::invoker()
{
    //start the datapool processing task
    taskDispatcher->performTask(processDatapoolTask);
}

void windowlistplugin::WindowListApplication::initializeDatapool()
{
    //get the path to the datapool description file, in order to load the datapool items
    configuration::PathValueHandle descriptionFile = getDatapoolDescription();
    if (!descriptionFile.valid())
    {
        return;
    }

    datapool->getLoader().load(descriptionFile->get());
    datapool->setInvoker(utils::Delegate<void()>::create<WindowListApplication, &WindowListApplication::invoker>(this));
    datapool->update();
    // read the initial values of all items in order to receive further notifications
    readAllDatapoolItems();
}

configuration::PathValueHandle windowlistplugin::WindowListApplication::getDatapoolDescription()
{
    // retrieve settings handle to provide access to the configuration items
    configuration::SettingsHandle settings = container->get<configuration::Settings>();
    if (!settings.valid())
    {
        return configuration::PathValueHandle();
    }

    // get the path to the model
    configuration::StringScalarHandle basePath = settings->getItem<configuration::StringScalar>("gtf.model.default.path");
    if (!basePath.valid())
    {
        return configuration::PathValueHandle();
    }

    // retrieve the configuration value of the "datapoolFile" item in the model.json
    configuration::PathValueHandle filePath = settings->getItem<configuration::PathValue>("datapoolFile");
    if (!filePath.valid())
    {
        return configuration::PathValueHandle();
    }
    // set the complete path to the datapool description file
    filePath->setBasePath(basePath->get());
    return filePath;
}

void windowlistplugin::WindowListApplication::readAllDatapoolItems()
{
    container::Array<uint32_t> items;

    bool canRetreiveItems = datapool->getAllItems(items);
    if (!canRetreiveItems)
    {
        return;
    }

    for (container::Array<uint32_t>::Iterator it = items.begin(), e = items.end(); it != e; ++it)
    {
        ValueHandle value = datapool->getItemByIndex<Value>(*it);

        // avoid reading items without a valid value, e.g. scripts
        if (!value)
        {
            continue;
        }

        value->read();
    }
}

void windowlistplugin::WindowListApplication::stop()
{
    datapool.reset();
}

void windowlistplugin::WindowListApplication::processDatapoolItems()
{
    container::Array<uint32_t> notifications;
    datapool->update();
    bool canRetrieveNotifications = datapool->getNotifications(notifications);

    if (!canRetrieveNotifications)
    {
        return;
    }
    for (container::Array<uint32_t>::Iterator it = notifications.begin(), e = notifications.end(); it != e; ++it)
    {
        switch (*it)
        {
            // update the windowed list when the datapool items bellow change
        case LOCAL_DP_ID_WindowPosition:
        case LOCAL_DP_ID_WindowSize:
        case LOCAL_DP_ID_WindowListSize:
            updateWindowList();
        default:
            break;
        }
    }
    // commit all changed properties
    datapool->commit();
}

void windowlistplugin::WindowListApplication::updateWindowList()
{
    int32_t listLength = 0, windowSize = 0, windowPosition_value = 0;

    char tmpBufferUTF8[50];

    // read the window position
    IntegerValueHandle windowPosition = datapool->getItemByIndex<IntegerValue>(LOCAL_DP_ID_WindowPosition);

    if (windowPosition.valid())
    {
        windowPosition->get(windowPosition_value);
    }

    // read the window size
    IntegerValueHandle  windowSizeHandle = datapool->getItemByIndex<IntegerValue>(LOCAL_DP_ID_WindowSize);

    if (windowSizeHandle.valid())
    {
        windowSizeHandle->get(windowSize);
    }

    // read size of the list
    IntegerValueHandle  listLengthHandle = datapool->getItemByIndex<IntegerValue>(LOCAL_DP_ID_WindowListSize);

    if (listLengthHandle.valid())
    {
        listLengthHandle->get(listLength);
    }

    StringListHandle windowList = datapool->getItemByIndex<StringList>(LOCAL_DP_ID_myReceivedWindowList);

    if (!windowList.valid())
    {
        return;
    }

    // set the type of the list to Windowed
    windowList->reset(List::Mode::Windowed);


    // resize the list
    windowList->resize(listLength);

    //create a window
    WindowHandle windowHandle = windowList->createWindow();

    if (!windowHandle.valid())
    {
        return;
    }

    // set the corresponding position and size of the window
    windowHandle->set(windowPosition_value, windowSize);

    uint32_t tmpLength = MIN(static_cast<uint32_t>(listLength), static_cast<uint32_t>(windowPosition_value + windowSize));

    // update the window content
    for (uint32_t index = windowPosition_value; index < tmpLength; index++)
    {
        switch (index)
        {
        case 0:
        {
            windowList->setElement(index, GTF_VERSION_STRING);
            break;
        }
        case 1:
        {
            windowList->setElement(index, GTF_VERSION_COMPANY);
            break;
        }

        case 2:
        {
            windowList->setElement(index, GTF_PRODUCT_NAME_STRING);
            break;
        }
        default:
        {
            sprintf(tmpBufferUTF8, "line %d", index);
            windowList->setElement(index, tmpBufferUTF8);
            break;
        }
        }
    }
}
